/*
SerialDisplay.h - Library for Sparkfunk serialdisplay
Version 2.24

by Cordier Yves mai 2017 Version du 11/02/2020
release :
V1.1 : 01/2015
V2.0 : 11/04/2016 adapted to ARM Arduino boards the 11/04/2016 (tested on due and uno)
V2.01 : 17/04/2016 Clreoln() added
V2.02 : 24/04/2016 Clreoln() debugged
V2.1  : 14/05/2016 harward serial connection allowed
V2.11 : 10/05/2017 correction to fix bug on due
V2.12 : 14/05/2017 type unsigned long long and long long added to print functions
V2.13 : 15/05/2017 correction of bug on V2.12 about long long 
V2.14 : 16/05/2017 simplification and producing more compact code
				added possibility of printing zeros before significant figures
				in print(n,base) could be write print (n,base,true) to do so;
V2.2  : 18/05/2017 rewriting of almost half of the librarie using C++ facilities
					accepting strings with \ characteres
					\n \t \v \b \r \f \a \\ \' \" \? \0
v2.21 : 19/05/2017 using pgm memory to store large arrays by /avr/pgmspace.h librairie
				Correction de la commande Begin qui possède deux parametres (vitesse, reinit)
				vitesse est de 9600 bds par defaut et reinit est true par defaut
				en mode final (appli stabilisée), il vaut mieux utiliser reinit à false
v2.22 :	19/10/2017  bug of cleroln fixed		
v2.23 : added functions to sleep and to awake the display when this one's power supply is connected to an output pin 	
v2.24 : minors bugs corrected to avoid warning's compiler
v2.3  :	23/03/2020 add scientific mode for float and double printing

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/******************************************************************************
* Includes
******************************************************************************/
#include "SerialDisplay.h"

// | Divers constantes pour gérer l'afficheur serie|
// =================================================
#define Ctrl_j 10
#define Ctrl_k 11
#define Ctrl_l 12
#define Ctrl_m 13 // code pour passe à 9600 bauds
#define Ctrl_n 14
#define Ctrl_o 15
#define Ctrl_p 16
#define Ctrl_q 17
#define Ctrl_r 18
#define bauds_9600 Ctrl_m
#define Car_cmd 254
#define Car_cmd_special 124 
#define Clear_lcd 01
#define CursRight_lcd 0x14
#define CursLeft_lcd 0x10
#define ScrollRight_lcd 0x1C
#define ScrollLeft_lcd 0x18
#define Displayon_lcd 0x0C
#define Displayoff_lcd 0x08
#define Underlineon_lcd 0x0E
#define Underlineoff_lcd 0x0C
#define Blinkon_lcd 0x0D
#define Blinkoff_lcd 0x0F // a verifier

// les vitesses de transmission sur l'afficheur

const long SerialDisplay_vitesse_afficheur [6] = {
	2400, 4800, 9600, 14400, 19200, 38400
};

// implementation de l'objet SerialOut
// Le constructeur par defaut
SerialOut::SerialOut()
{ 
}

// Le constructeur alternatif
SerialOut::SerialOut(byte tx)

{ 
	transmitPin=tx;
	baudRate = 9600;
	pinMode(transmitPin,OUTPUT);
	serialSet=false;
	hardSerial=false;
}
// le constructeur précisant la sortie serie materielle
SerialOut::SerialOut(HardwareSerial *ser)

{ 
	_serial=ser;
	hardSerial=true;
	baudRate = 9600;
	serialSet=false;
}
// Calcul une durée entre T1 et T2 T1<T2
unsigned long SerialOut::deltaT(unsigned long T1,unsigned long T2)
{
	if (T2>T1) 
	{return (T2-T1);}
	else
	{return ((4294967295-T1)+T2);}
}

// Reglage des parametres de transmission
void SerialOut::setSerial()
{
	if (!serialSet)	
	{   // procedure adaptative de recherche du temps necessaire 
		// pour mettre à 1 ou 0 une sortie digitale
		unsigned long T1,T2,Dt,T;
		T=0;
		// train de bits alterné pour le test
		boolean bits[10]={true,false,true,false,true,false,true,false}; 
		
		
		// Correction pour prendre en compte les cartes due
		// on sauve les registres d'interruption et on masque les interruptions
		#if defined( SAM ) // due generic
		uint32_t old_primask = __get_PRIMASK();
		__disable_irq();
		#else // arm proc
		uint8_t oldSREG = SREG; // On sauve le registre d'interruption
		cli(); // masquage des interruptions
		#endif
		// recherche du temps pour afficher une sortie 
		T1=micros();
		for (int i=0; i<100; i++)
		{
			for (byte j = 0; j<10; j++) 
			{
				digitalWrite(transmitPin,bits[j]);
				delayMicroseconds(T);
			}
		}
		T2=micros();
		// Correction pour prendre en compte les cartes due
		// on remet en place le masque des interruptions
		#if defined( SAM ) // due generic
		__set_PRIMASK( old_primask );
		#else // arm proc
		SREG=oldSREG; // on remet en place le registre d'interruption
		#endif
		
		Dt=deltaT(T1,T2);
		correctionAttente=-(float(Dt)/1000.); // en microsecondes généralement cela donne 1.22 micro secondes (equivalent de 20 instructions)
		// pour le deboggage
		/*
		Serial.println(T1);
		Serial.println(T2);
		Serial.println(Dt); 
		Serial.println(correctionAttente);
		for (int i=0; i<6; i++)
		{
			Serial.print(SerialDisplay_vitesse_afficheur[i]); Serial.print("  : ");
			Serial.print((Dt/5000.)/(1000000.0/SerialDisplay_vitesse_afficheur[i])*100);
			Serial.println("%");
		}
		
		*/
		// correction de temps pour prendre en compte le delai de basculement des sorties
		serialSet=true;
	}
	
	
}

// Initialisation de la sortie serie
void SerialOut::begin(long vitesse)
{ 
	if (hardSerial)
	{
		_serial->begin(vitesse);
	}
	else
	{
		if (!serialSet)
		{setSerial(); }

		delaiEntreBits=round(1000000.0/vitesse+correctionAttente);
		// pour le deboggage;
		/*Serial.print(vitesse); 
		Serial.print(" : delai ");
		Serial.print(1000000.0/vitesse+correctionAttente);
		Serial.print(" brut ");
		Serial.print(delaiEntreBits); 
		Serial.print(" retenu ");
		Serial.print(" micro s temps pour un bit : ");
		Serial.println(1000000./vitesse);
		*/
	}
	baudRate=vitesse;
	
}

// La clef de voute du systeme : ecrire un caractère sur la sortie serie
void SerialOut::write(char b)
{ 
	if (hardSerial)
	{
		_serial->write(b);
		return;
	}
	// Le train de bits correspondant à 0xFF
	static boolean bits[10] = {false,true,true,true,true,true,true,true,true,true}; 
	// 1 bit de start (0) , 8 bits données, 1 bit de stop soit 10 bits
	if (baudRate == 0)
	return;  // par securité
	// preparation du train de bits, la partie variable correspondant à la donnée
	for (byte i=0; i<8; i++)
	
	{
		bits[i+1]=((byte(b) & (0x01<<(i))) >0);
	}

	// Correction pour prendre en compte les cartes due
	// on sauve les registres d'interruption et on masque les interruptions
	#if defined( SAM ) // due generic
	uint32_t old_primask = __get_PRIMASK();
	__disable_irq();
	#else // arm proc
	uint8_t oldSREG = SREG; // On sauve le registre d'interruption
	cli(); // masquage des interruptions
	#endif
	
	// envoi du train de bits complet
	for (byte i=0;i<10;i++) 
	{
		digitalWrite(transmitPin,bits[i]);
		delayMicroseconds(delaiEntreBits);
	}
	// Correction pour prendre en compte les cartes due
	// on remet en place le masque des interruptions
	#if defined( SAM ) // due generic
	__set_PRIMASK( old_primask );
	#else // arm proc
	SREG=oldSREG; // on remet en place le registre d'interruption
	#endif
	
	if ((b == Car_cmd_special) || (byte(b) == Car_cmd)) 
	{delayMicroseconds (10000000/baudRate);} // ajouté pour gérer la rapidité des cartes ARM
}




// byte commande_vitesse_afficheur[6] = {Ctrl_k, Ctrl_l, Ctrl_m, Ctrl_n, Ctrl_o, Ctrl_p};



// Le constructeur serie soft
SerialDisplay::SerialDisplay(byte tx) : SerialOut(tx)

{	
	cursX=0;
	_Max_X=16;
	_Max_Y=2;
	
}

// Le constructeur serie soft ameliore
SerialDisplay::SerialDisplay(byte tx, byte sizex, byte sizey) : SerialOut(tx)

{
	cursX=0;
	_Max_X=sizex;
	_Max_Y=sizey;
	if ((_Max_X!=16) &(_Max_X!=20)) {_Max_X=16;}; // recadrage en cas de valeurs fantaisistes
	if ((_Max_Y!=2) & (_Max_Y!=4)) {_Max_Y=2;}; // recadrage en cas de valeurs fantaisistes
	
}
// Le constructeur serie hard simple
SerialDisplay::SerialDisplay(HardwareSerial *ser) : SerialOut(ser)

{	
	cursX=0;
	_Max_X=16;
	_Max_Y=2;
	
	
}
// Le constructeur serie hard ameliore
SerialDisplay::SerialDisplay(HardwareSerial *ser, byte sizex, byte sizey) : SerialOut(ser)

{
	cursX=0;
	_Max_X=sizex;
	_Max_Y=sizey;
	if ((_Max_X!=16) &(_Max_X!=20)) {_Max_X=16;}; // recadrage en cas de valeurs fantaisistes
	if ((_Max_Y!=2) & (_Max_Y!=4)) {_Max_Y=2;}; // recadrage en cas de valeurs fantaisistes
	
}
void SerialDisplay::Clrscr()
{
	write(char(Car_cmd));
	write(char(Clear_lcd));
	cursX=0;
	
}

void SerialDisplay::set_size_lcd()
{
	write(Car_cmd_special);
	if (_Max_X==20) {write(3);} // 20 caracteres par ligne
	else {write(4);}; // par defaut 18
	delay(100);
	write(Car_cmd_special);
	if (_Max_Y==4) {write(5);} // 4 lignes de texte
	else {write(6);}; // par defaut 2
	delay(100); // pour pouvoir ecrire en mémoire EEPROM sans risques
}
// Gestion des curseurs sur l'afficheur
// ************************************
// positionne le curseur à x,y (le point en haut à gauche à comme coordonnées (1,1)

void SerialDisplay::Posxy(byte x, byte y)
{ 
	byte code = 128; // bit de poids fort à 1
	y=y % (_Max_Y+1); // enroulement des lignes en fonction du nombre de lignes
	x=x % (_Max_X+1); //  enroulement des colonnes en fonction du nombre de colonnes
	if (_Max_X==16) // cas 16 colonnes
	{
		switch (y)
		{
		case 1 : {code =code+(x-1); break;}
		case 2 : {code =code+64+(x-1); break;}
		case 3 : {code =code+16+(x-1); break;}
		case 4 : {code =code+80+(x-1); break;}
		}
	}
	else // cas 20 colonnes
	{	switch (y)
		{
		case 1 : {code =code+(x-1); break;}
		case 2 : {code =code+64+(x-1); break;}
		case 3 : {code =code+20+(x-1); break;}
		case 4 : {code =code+84+(x-1); break;}
		}
	}
	write(char(Car_cmd));
	write(char(code));
	cursX=x-1; // mise à jour du curseur logiciel
}
// ***************************************************************
// efface la fin de la ligne en cours et passe à la ligne suivante

void SerialDisplay::Clreoln()
{
	if (cursX==0) print(' ');
	while (cursX)
	{
		print(' ');
	}
}

// Initialisation de l'afficheur
// *****************************
void SerialDisplay::begin(long vitesse, boolean reinit)
{  
	if (reinit)
	{
		// Tentative au boot de l'afficheur de le mettre a 9600 bauds
		SerialOut::begin(9600); 
		write(char(Ctrl_r));
		
		// On scan tous les modes possibles pour le remettre à 9600 bauds
		for (int i = 0; i < 6; i++)
		{
			SerialOut::begin(SerialDisplay_vitesse_afficheur[i]);
			delay(100);
			write(char(Car_cmd_special));
			write(char(Ctrl_m) ); // Reset l'afficheur à 9600		
		}
		SerialOut::begin(9600); // Remise de la serie en 9600 bauds
		delay(100);
		
		// On bascule l'afficheur à la vitesse demandée
		for (int i = 0; i < 6; i++)
		{ 		
			if (SerialDisplay_vitesse_afficheur[i] == vitesse)			
			{
				// Commande de l'afficheur
				write(char(Car_cmd_special));
				write(char(Ctrl_k+i));
				// On bascule la série au bon mode
				SerialOut::begin(vitesse);
				delay(100);
			}
		}
	}
	else
	{
		// On bascule la série au bon mode et on suppose l'afficheur bien configuré
		SerialOut::begin(vitesse);
	}
	set_size_lcd();	
	Clrscr();
	delay(100);
	Backlight(30);
	delay(100);
	print(F(Version)); println(" Display OK");
	print(baudRate);
	print (F(" bds"));
	delay(3000);
	Clrscr();
}
void SerialDisplay::CursRight()
{
	write(char(Car_cmd));
	write(char(CursRight_lcd));
	cursX++;
	cursX%= _Max_X;
}
void SerialDisplay::CursLeft()
{
	write(char(Car_cmd));
	write(char(CursLeft_lcd));
	cursX--;
	cursX %= _Max_X;
}
void SerialDisplay::ScrollRight()
{
	write(char(Car_cmd));
	write(char(ScrollRight_lcd));
}
void SerialDisplay::ScrollLeft()
{
	write(char(Car_cmd));
	write(char(ScrollLeft_lcd));
};
void SerialDisplay::Displayon()
{
	write(char(Car_cmd));
	write(char(Displayon_lcd));
}
void SerialDisplay::Displayoff()
{
	write(char(Car_cmd));
	write(char(Displayoff_lcd));
}
void SerialDisplay::Underlineon()
{
	write(char(Car_cmd));
	write(char(Underlineon_lcd));
}
void SerialDisplay::Underlineoff()
{
	write(char(Car_cmd));
	write(char(Underlineoff_lcd));
}
void SerialDisplay::Blinkingon()
{
	write(char(Car_cmd));
	write(char(Blinkon_lcd));
}
void SerialDisplay::Blinkingoff()
{
	write(char(Car_cmd));
	write(char(Blinkoff_lcd));
}
void SerialDisplay::Backlight(byte i)
{
	if (i>30) {i=30;}
	write(char(124));
	write(char(127+i));
}

boolean SerialDisplay::traiteCaractereSpeciaux(char s) // traitement des caractères spéciaux d'afichage

{switch (s)
	{
	case 'n' : println(); return true;break; // saut de ligne et retour chariot
	case 'b' : CursLeft(); return true;break; // recule d'un caractère
	case '\\' : print('\\'); return true;break; // l'antislash
	case '\'' : print('\''); return true; break; // '
	case '\"' : print('\"'); return true; break; // '
	case '?' : print('?'); return true;break; // point d'interrogation
	case '\0' : return false;break; // caractere nul de fin
	case 'a' : // alarme !!
	case 't' : // tabulation horizontale!!
	case 'v' : return true; break; // tabulation verticale
	case 'f' : Clrscr();return true;break; // saut de page
	case 'r' : cursX=0;return true;break; // retour chariot
	}
	return true;

}

void SerialDisplay::print(const char *s)
{ 
	while (*s)
	{
		
		if (*s!='\\') 
		{
			print(*s++); // imprime le caractère en mode caractère}
		}
		else
		{
			s++; // avance d'un caractère
			if (traiteCaractereSpeciaux(*s)) s++;
			
		}
		
	}
}

void SerialDisplay::print(String s)
{ int i=0;
	while (s[i])
	{
		if (s[i]!='\\') 
		{
			print(s[i]); // imprime le caractère en mode caractère}
			i++;
		}
		else
		{
			i++; // avance d'un caractère
			if (traiteCaractereSpeciaux(s[i])) i++;
		}
		
	}
}


void SerialDisplay::print(char c)
{ 
	if ((c>=char(32))&(c<=char(126))& (c!=char(124))) 
	{write(c);}
	else 
	{write(char(32));}
	cursX++;
	cursX= cursX % _Max_X; // avance le curseur logiciel de 1
}

// impression en base 10 par defaut 
void SerialDisplay::print(byte b)
{
	printNumber(b,1);
}

void SerialDisplay::print(int n)
{
	if (n < 0) {
		print('-');
		n = -n;
	}
	printNumber(n,2);
}

void SerialDisplay::print(unsigned int n)
{
	printNumber(n,2);
}

void SerialDisplay::print(long n)
{
	if (n < 0) {
		print('-');
		n = -n;
	}
	printNumber( n,4);
}

void SerialDisplay::print(unsigned long n)
{
	printNumber( n,4);
}

void SerialDisplay::print(long long n)
{
	if (n < 0) {
		print('-');
		n = -n;
	}
	printNumber ( n,8);
}

void SerialDisplay::print(unsigned long long n)
{
	printNumber(n,8);
}

// impression dans une base donnée particuliere

void SerialDisplay::print(byte n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber(n,1,base,zero);
}

void SerialDisplay::print(int n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber((unsigned int)n,2,base,zero);
}

void SerialDisplay::print(unsigned int n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber(n,2,base,zero);
}

void SerialDisplay::print(long n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber((unsigned long)n,4,base,zero);
}

void SerialDisplay::print(unsigned long n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber(n,4,base,zero);
}
void SerialDisplay::print(long long n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber((unsigned long long)n,8,base,zero);
}

void SerialDisplay::print(unsigned long long n, int base,boolean zero)
{
	if (base <2 )
	print((char) n);
	else
	printNumber(n,8,base,zero);
}




void SerialDisplay::println(void)
{
	while (cursX!=0){print(char(32));}; // On complete la ligne de blanc et on passe à la ligne automatiquement
}

void SerialDisplay::println(char c)
{
	print(c);
	println();  
}

void SerialDisplay::println(const char c[])
{
	print(c);
	println();
}

void SerialDisplay::println(String s)
{
	print(s);
	println();
}
// en base 10
void SerialDisplay::println(byte b)
{
	print(b);
	println();
}

void SerialDisplay::println(int n)
{
	print(n);
	println();
}

void SerialDisplay::println(unsigned int n)
{
	print(n);
	println();
}

void SerialDisplay::println(long n)
{
	print(n);
	println();  
}

void SerialDisplay::println(unsigned long n)
{
	print(n);
	println();  
}

void SerialDisplay::println(long long n)
{
	print(n);
	println();  
}

void SerialDisplay::println(unsigned long long n)
{
	print(n);
	println();  
}


void SerialDisplay::println(byte b, int base,boolean zero)
{
	print(b, base,zero);
	println();
}

void SerialDisplay::println(int n,int base,boolean zero)
{
	print(n, base,zero);
	println();
}

void SerialDisplay::println(unsigned int n,int base,boolean zero)
{
	print(n, base,zero);
	println();
}

void SerialDisplay::println(long n,int base,boolean zero)
{
	print(n, base,zero);
	println();  
}

void SerialDisplay::println(unsigned long n,int base,boolean zero)
{
	print(n, base,zero);
	println();  
}

void SerialDisplay::println(long long n,int base,boolean zero)
{
	print(n, base,zero);
	println();  
}

void SerialDisplay::println(unsigned long long n,int base,boolean zero)
{
	print(n, base,zero);
	println();  
}

// impression des réels
void SerialDisplay::print(double n,int d,boolean scienti)
{
	if (scienti) printScienti( n, d); 
	else printFloat(n,d);
}

void SerialDisplay::print(float n,int d,boolean scienti)
{
	if (scienti) printScienti(n, d); 
	else printFloat(n,d);
}

void SerialDisplay::println(double n, int d,boolean scienti)
{
	if (scienti) printScienti(n, d); 
	else printFloat(n,d);
	println();
}

void SerialDisplay::println(float n, int d,boolean scienti)
{
	if (scienti) printScienti( n, d); 
	else printFloat(n,d);
	println();
}


// Private Methods /////////////////////////////////////////////////////////////

#ifndef _progmem
const  byte taillebuffer1[] ={8,6,4,4,4,3,3,3,3,3,3,3,3,3,2};
const  byte taillebuffer2[] ={16,11,8,7,7,6,6,6,5,5,5,5,5,5,4};
const  byte taillebuffer4[] ={32,21,16,14,13,12,11,11,10,10,9,9,9,9,8};
const  byte taillebuffer8[] ={64,41,32,28,25,23,22,21,20,19,18,18,17,17,16};
#endif

#ifdef _progmem
const  byte taillebuffer1[] PROGMEM ={8,6,4,4,4,3,3,3,3,3,3,3,3,3,2};
const  byte taillebuffer2[] PROGMEM ={16,11,8,7,7,6,6,6,5,5,5,5,5,5,4};
const  byte taillebuffer4[] PROGMEM ={32,21,16,14,13,12,11,11,10,10,9,9,9,9,8};
const  byte taillebuffer8[] PROGMEM ={64,41,32,28,25,23,22,21,20,19,18,18,17,17,16};
#endif
// pré calcul de la boucle
// for (int i=2-2; i<=16-2; i++)
// {
// taillebuffer1 [base-2]=int(ceil(8*1*log(2)/log(base))); 
// taillebuffer2 [base-2]=int(ceil(8*2*log(2)/log(base))); 
// taillebuffer4 [base-2]=int(ceil(8*4*log(2)/log(base))); 
// taillebuffer8 [base-2]=int(ceil(8*8*log(2)/log(base))); 
// correspond au nombre de digits necessaires pour afficher un long dans une base donnée
// }

void SerialDisplay::printNumber(unsigned long long n,byte taille, byte base,boolean zero)
{   
	if (base>16) {print('#'); return;} // on limite la base 0, 2..16
	byte len=8; // par defaut le maximum
	switch (taille) // pour dimentionner le buffer de caractères maxi en fonction de la taille réelle en octet de la donnée
	{ 
		#ifndef _progmem
	case 1 : len=taillebuffer1[base-2]; break;
	case 2 : len=taillebuffer2[base-2]; break;
	case 4 : len=taillebuffer4[base-2]; break;
	case 8 : len=taillebuffer8[base-2]; break;
		#endif
		#ifdef _progmem
	case 1 : len=pgm_read_byte(&(taillebuffer1[base-2])); break;
	case 2 : len=pgm_read_byte(&(taillebuffer2[base-2])); break;
	case 4 : len=pgm_read_byte(&(taillebuffer4[base-2])); break;
	case 8 : len=pgm_read_byte(&(taillebuffer8[base-2])); break;
		#endif
	}
	unsigned char buf[len]; // la taille du buffer necessaire pour afficher
	// on rempli le buffer des digits significatifs
	byte i = 0; // poids faible
	do
	{
		buf[i] = n % base; 
		n /= base;
		i++;
	}
	while (n > 0) ;// on regarde le reste pas à pas
	// on passe au moins une fois => i>0
	if (zero) 
	{
		while (i<len) // dans ce cas on rempli de 0 le reste du buffer
		{
			buf[i]=0; 
			i++;
		} 
	}
	// i vaut le maxi des poids calculés 
	for (; i > 0; i--) // on affiche les caractères du buffer du poids fort au poids faible
	{
		print((char) (buf[i - 1] < 10 ? '0' + buf[i - 1] : 'A' + buf[i - 1] - 10)); // on imprime les caracteres 
		// en utilisant les procedures de plus bas niveau
	}
}

void SerialDisplay::printFloat(double number, uint8_t digits)
{
	// Handle negative numbers
	if (number < 0.0)
	{
		print('-');
		number = -number;
	}

	// Round correctly so that print(1.999, 2) prints as "2.00"
	double rounding = 0.5;
	for (uint8_t i=0; i<digits; ++i) // compute the smallest part of rounding depends on digis
	rounding /= 10.0;

	number += rounding; // increase the number by this small part

	// Extract the integer part of the number and print it
	unsigned long int_part = (unsigned long)number;
	double remainder = number - (double)int_part;
	print(int_part);

	// Print the decimal point, but only if there are digits beyond
	if (digits > 0)
	print('.');

	// Extract digits from the remainder one at a time
	while (digits-- > 0)
	{
		remainder *= 10.0;
		int toPrint = int(remainder);
		print(toPrint);
		remainder -= toPrint;
	}
}

void SerialDisplay::printScienti(double d, uint8_t dec ) // print a double scientific methode with n digits decimal 
{
	if (d<0.0) 
	{
		print('-');
		d=-d;
	}
	int exp =0;
	while (d>=10) {exp++; d/=10;} // search of exponent 10^x if >0
	while ((d<1)&&(d>0)) {exp--; d*=10;} // search of exponent 10^-x if <0
	// now d in in range [0,10[
	
	// Round correctly so that print(1.999, 2) prints as "2.00"
	double rounding = 0.5;
	for (uint8_t i=0; i<dec; ++i) // compute the smallest part of rounding depends on digis
	rounding /= 10.0;

	d += rounding; // increase the number by this small part
	// verifying the range
	if (d>=10) {d/=10; exp++;}
	
	byte ent=d; // ent part of d  <10
	print(ent);
	d=d-ent; // decimal part of d
	if (dec>0)
	{
		print('.');
		for (byte i=0; i<dec; i++) // print decimal part
		{
			d=d*10; 
			byte dig=byte(d);
			print(char(48+dig));
			d-=dig; 
		}	
		
	}
	
	// print exponent "e" {+|-} {dig (2|3)}
	print('e');
	if (exp>=0) print('+'); else print('-');
	exp=abs(exp);
	if (exp <10) print('0'); // character '0'
	print(exp);
}

void SerialDisplay::awake(long speed)
{
	if (powerPin>=0)
	{
		pinMode(powerPin,OUTPUT);
		if (digitalRead(powerPin))
		{
			digitalWrite(powerPin,false);
			delay(100);
		}
		
		digitalWrite(brocheOn, true); // On alimente la broche
		delay(100);
		begin(speed,true);
		
	}
}
void SerialDisplay::sleep()
{
	if (powerPin>0)
	{
		pinMode(powerPin,OUTPUT);
		digitalWrite(powerPin,false);
	}
}

void SerialDisplay::setPowerPin(int p)
{
	powerPin=p;
}	

